/*
 
 This file is part of Subconnector.
 
 Subconnector is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 Subconnector is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with Subconnector. If not, see <http://www.gnu.org/licenses/>.
 
 Copyright 2012, 2013 Alejandro Celaya Alastrué
 
 */

package net.subclient.subsonic;

import static org.junit.Assert.*;

import java.io.IOException;
import java.net.URL;
import java.net.URLDecoder;

import net.subclient.subsonic.deserializers.AlbumsDeserializer;
import net.subclient.subsonic.deserializers.ChannelInfoDeserializer;
import net.subclient.subsonic.deserializers.DirectoryDeserializer;
import net.subclient.subsonic.deserializers.GetAlbumsResponseDeserializer;
import net.subclient.subsonic.deserializers.GetMusicFoldersResponseDeserializer;
import net.subclient.subsonic.deserializers.GetPlaylistsResponseDeserializer;
import net.subclient.subsonic.deserializers.GetPodcastsResponseDeserializer;
import net.subclient.subsonic.deserializers.GetRandomSongsResponseDeserializer;
import net.subclient.subsonic.deserializers.IndexInfoDeserializer;
import net.subclient.subsonic.deserializers.IndexesDeserializer;
import net.subclient.subsonic.deserializers.MusicFoldersDeserializer;
import net.subclient.subsonic.deserializers.PlaylistInfoDeserializer;
import net.subclient.subsonic.deserializers.PlaylistsDeserializer;
import net.subclient.subsonic.deserializers.PodcastsDeserializer;
import net.subclient.subsonic.deserializers.RandomSongsDeserializer;
import net.subclient.subsonic.deserializers.SearchResponseDeserializer;
import net.subclient.subsonic.deserializers.SearchResultDeserializer;
import net.subclient.subsonic.mappings.Albums;
import net.subclient.subsonic.mappings.ChannelInfo;
import net.subclient.subsonic.mappings.ChildInfo;
import net.subclient.subsonic.mappings.Directory;
import net.subclient.subsonic.mappings.FolderInfo;
import net.subclient.subsonic.mappings.IndexInfo;
import net.subclient.subsonic.mappings.Indexes;
import net.subclient.subsonic.mappings.MusicFolders;
import net.subclient.subsonic.mappings.PlaylistInfo;
import net.subclient.subsonic.mappings.Playlists;
import net.subclient.subsonic.mappings.Podcasts;
import net.subclient.subsonic.mappings.RandomSongs;
import net.subclient.subsonic.mappings.SearchResult;
import net.subclient.subsonic.responses.GetAlbumsResponse;
import net.subclient.subsonic.responses.GetIndexesResponse;
import net.subclient.subsonic.responses.GetLicenseResponse;
import net.subclient.subsonic.responses.GetMusicDirectoryResponse;
import net.subclient.subsonic.responses.GetMusicFoldersResponse;
import net.subclient.subsonic.responses.GetPlaylistResponse;
import net.subclient.subsonic.responses.GetPlaylistsResponse;
import net.subclient.subsonic.responses.GetPodcastResponse;
import net.subclient.subsonic.responses.GetPodcastsResponse;
import net.subclient.subsonic.responses.GetRandomSongsResponse;
import net.subclient.subsonic.responses.SearchResponse;
import net.subclient.subsonic.responses.SubsonicResponse;
import net.subclient.subsonic.util.FileUtils;
import net.subclient.subsonic.util.Version;

import org.junit.Before;
import org.junit.Test;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;


public class SubsonicResponsesTest {
	
	private Gson gson = null;
	
	/**
	 * Sets up the JSON parser
	 * @throws Exception
	 */
	@Before
	public void setUp() throws Exception {
		this.gson = new GsonBuilder()
		  .registerTypeHierarchyAdapter(GetMusicFoldersResponse.class	, new GetMusicFoldersResponseDeserializer())
		  .registerTypeHierarchyAdapter(MusicFolders.class				, new MusicFoldersDeserializer())
		  .registerTypeHierarchyAdapter(IndexInfo.class					, new IndexInfoDeserializer())
		  .registerTypeHierarchyAdapter(SearchResponse.class			, new SearchResponseDeserializer())
		  .registerTypeHierarchyAdapter(SearchResult.class				, new SearchResultDeserializer())
		  .registerTypeHierarchyAdapter(GetPlaylistsResponse.class		, new GetPlaylistsResponseDeserializer())
		  .registerTypeHierarchyAdapter(Playlists.class					, new PlaylistsDeserializer())
		  .registerTypeHierarchyAdapter(PlaylistInfo.class				, new PlaylistInfoDeserializer())
		  .registerTypeHierarchyAdapter(GetAlbumsResponse.class			, new GetAlbumsResponseDeserializer())
		  .registerTypeHierarchyAdapter(Albums.class					, new AlbumsDeserializer())
		  .registerTypeHierarchyAdapter(GetPodcastsResponse.class		, new GetPodcastsResponseDeserializer())
		  .registerTypeHierarchyAdapter(Podcasts.class					, new PodcastsDeserializer())
		  .registerTypeHierarchyAdapter(ChannelInfo.class				, new ChannelInfoDeserializer())
		  .registerTypeHierarchyAdapter(Directory.class					, new DirectoryDeserializer())
		  .registerTypeHierarchyAdapter(Indexes.class					, new IndexesDeserializer())
		  .registerTypeHierarchyAdapter(GetRandomSongsResponse.class	, new GetRandomSongsResponseDeserializer())
		  .registerTypeHierarchyAdapter(RandomSongs.class				, new RandomSongsDeserializer())
		  .create();
	}
	
	/**
	 * @see net.subclient.subsonic.responses.SubsonicResponse
	 */
	@Test
	public void testSubsonicResponse() {
		String mockFile = "SubsonicResponseMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			SubsonicResponse resp = this.gson.fromJson(responseText, SubsonicResponse.class);
			Version expectedVersion = new Version(1, 8);
			
			//Assertions
			assertEquals(SubsonicResponse.STATUS_OK, resp.getStatus());
			assertNotSame(SubsonicResponse.STATUS_FAILED, resp.getStatus());
			assertTrue(expectedVersion.compareTo(resp.getVersion()) == 0);
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		}
	}
	
	/**
	 * see net.subclient.subsonic.responses.GetLicenseResponse
	 */
	@Test
	public void testGetLicenseValid() {
		String mockFile = "GetLicenseResponseValidMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetLicenseResponse resp = this.gson.fromJson(responseText, GetLicenseResponse.class);
			
			//Assertions
			boolean expectedValidity	= true;
			String expectedEmail		= "foobar@gmail.com",
				   expectedDate 		= "2012-04-10T22:54:44",
				   expectedKey			= "8843d7f92416211de9ebb963ff4ce28125932878";
			assertEquals(expectedValidity, resp.getSubsonicLicense().isValid());
			assertEquals(expectedEmail, resp.getSubsonicLicense().getEmail());
			assertEquals(expectedDate, resp.getSubsonicLicense().getDate());
			assertEquals(expectedKey, resp.getSubsonicLicense().getKey());
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	/**
	 * see net.subclient.subsonic.responses.GetLicenseResponse
	 */
	@Test
	public void testGetLicenseInvalid() {
		String mockFile = "GetLicenseResponseInvalidMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetLicenseResponse resp = this.gson.fromJson(responseText, GetLicenseResponse.class);
			
			//Assertions
			boolean expectedValidity	= false;
			String expectedEmail		= "john.doe@domain.com",
				   expectedDate 		= "",
				   expectedKey 			= "";
			assertEquals(expectedValidity, resp.getSubsonicLicense().isValid());
			assertEquals(expectedEmail, resp.getSubsonicLicense().getEmail());
			assertEquals(expectedDate, resp.getSubsonicLicense().getDate());
			assertEquals(expectedKey, resp.getSubsonicLicense().getKey());
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	
	/**
	 * @see net.subclient.subsonic.responses.GetIndexesResponse
	 */
	@Test
	public void testGetIndexesResponseRegular() {
		String mockFile = "GetIndexesResponseRegularMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetIndexesResponse resp = this.gson.fromJson(responseText, GetIndexesResponse.class);
			long lastModifiedExpected = 1363553853000L;
			
			//Assertions
			assertEquals(lastModifiedExpected, resp.getIndexes().getLastModified());
			assertTrue(resp.getIndexes().getIndexesArray().size() == 17);
			IndexInfo infoD = resp.getIndexes().getIndexesArray().get(3);
			assertTrue(infoD.getName().equalsIgnoreCase("D"));
			assertTrue(infoD.getArtistsArray().size() == 4);
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	/**
	 * @see net.subclient.subsonic.responses.GetIndexesResponse
	 */
	@Test
	public void testGetIndexesResponseComplete() {
		String mockFile = "GetIndexesResponseCompleteMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetIndexesResponse resp = this.gson.fromJson(responseText, GetIndexesResponse.class);
			long lastModifiedExpected = 1363553853000L;
			
			//Assertions
			assertEquals(lastModifiedExpected, resp.getIndexes().getLastModified());
			assertTrue(resp.getIndexes().getIndexesArray().size() == 22);
			IndexInfo infoD = resp.getIndexes().getIndexesArray().get(6);
			assertTrue(infoD.getName().equalsIgnoreCase("G"));
			assertTrue(infoD.getArtistsArray().size() == 1);
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	/**
	 * @see net.subclient.subsonic.responses.GetIndexesResponse
	 */
	@Test
	public void testGetIndexesResponseEmpty() {
		String mockFile = "GetIndexesResponseEmptyMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetIndexesResponse resp = this.gson.fromJson(responseText, GetIndexesResponse.class);
			long lastModifiedExpected = 1363553853000L;
			
			//Assertions
			assertEquals(lastModifiedExpected, resp.getIndexes().getLastModified());
			assertTrue(resp.getIndexes().getIndexesArray().size() == 0);
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	
	/**
	 * @see net.subclient.subsonic.responses.GetMusicFoldersResponse
	 */
	@Test
	public void testGetMusicFoldersResponse() {
		String mockFile = "GetMusicFoldersResponseMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetMusicFoldersResponse resp = this.gson.fromJson(responseText, GetMusicFoldersResponse.class);
			
			//Assertions
			assertTrue(resp.getMusicFolders().getMusicFoldersArray().size() == 2);
			assertEquals(resp.getMusicFolders().getMusicFoldersArray().get(0).getName(), "Music");
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	/**
	 * @see net.subclient.subsonic.responses.GetMusicFoldersResponse
	 */
	@Test
	public void testGetMusicFoldersResponseEmpty() {
		String mockFile = "GetMusicFoldersResponseEmptyMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetMusicFoldersResponse resp = this.gson.fromJson(responseText, GetMusicFoldersResponse.class);
			
			//Assertions
			assertTrue(resp.getMusicFolders().getMusicFoldersArray().size() == 0);
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	/**
	 * @see net.subclient.subsonic.responses.GetMusicFoldersResponse
	 */
	@Test
	public void testGetMusicFoldersResponseSingleEmpty() {
		String mockFile = "GetMusicFoldersResponseSingleMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetMusicFoldersResponse resp = this.gson.fromJson(responseText, GetMusicFoldersResponse.class);
			
			//Assertions
			assertTrue(resp.getMusicFolders().getMusicFoldersArray().size() == 1);
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	
	/**
	 * @see net.subclient.subsonic.responses.GetMusicDirectoryResponse
	 */
	@Test
	public void testGetMusicDirectoryResponseMultipleAlbums() {
		String mockFile = "GetMusicDirectoryResponseMultipleAlbumsMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetMusicDirectoryResponse resp = this.gson.fromJson(responseText, GetMusicDirectoryResponse.class);
			
			//Assertions
			int albums 			= 0,
				expectedAlbums	= 8,
				songs 			= 0,
				expectedSongs 	= 0,
				expectedSize 	= expectedSongs + expectedAlbums;
			assertEquals(expectedSize, resp.getDirectory().getChildsArray().size());
			assertEquals("19", resp.getDirectory().getId());
			assertEquals("Alanis Morissette", resp.getDirectory().getName());
			//Only albums should be found
			for(ChildInfo child : resp.getDirectory().getChildsArray()) {
				if(child.isDir()) 	albums++;
				else				songs++;
			}
			assertEquals(expectedAlbums, albums);
			assertEquals(expectedSongs, songs);
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	/**
	 * @see net.subclient.subsonic.responses.GetMusicDirectoryResponse
	 */
	@Test
	public void testGetMusicDirectoryResponseMultipleSongs() {
		String mockFile = "GetMusicDirectoryResponseMultipleSongsMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetMusicDirectoryResponse resp = this.gson.fromJson(responseText, GetMusicDirectoryResponse.class);
			
			//Assertions
			int albums 			= 0,
				expectedAlbums	= 0,
				songs 			= 0,
				expectedSongs 	= 12,
				expectedSize 	= expectedSongs + expectedAlbums;
			assertEquals(expectedSize, resp.getDirectory().getChildsArray().size());
			assertEquals("46", resp.getDirectory().getId());
			assertEquals("Alanis Morissette - (1994) Jagged Little Pill", resp.getDirectory().getName());
			//Only songs should be found
			for(ChildInfo child : resp.getDirectory().getChildsArray()) {
				if(child.isDir()) 	albums++;
				else				songs++;
			}
			assertEquals(expectedAlbums, albums);
			assertEquals(expectedSongs, songs);
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	/**
	 * @see net.subclient.subsonic.responses.GetMusicDirectoryResponse
	 */
	@Test
	public void testGetMusicDirectoryResponseEmpty() {
		String mockFile = "GetMusicDirectoryResponseEmptyMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetMusicDirectoryResponse resp = this.gson.fromJson(responseText, GetMusicDirectoryResponse.class);
			
			//Assertions
			assertEquals(0, resp.getDirectory().getChildsArray().size());
			assertEquals("22", resp.getDirectory().getId());
			assertEquals("AC.DC", resp.getDirectory().getName());
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	/**
	 * @see net.subclient.subsonic.responses.GetMusicDirectoryResponse
	 */
	@Test
	public void testGetMusicDirectoryResponseMixed() {
		String mockFile = "GetMusicDirectoryResponseMixedMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetMusicDirectoryResponse resp = this.gson.fromJson(responseText, GetMusicDirectoryResponse.class);
			
			//Assertions
			int albums 			= 0,
				expectedAlbums	= 4,
				songs 			= 0,
				expectedSongs 	= 13,
				expectedSize 	= expectedSongs + expectedAlbums;
			assertEquals(expectedSize, resp.getDirectory().getChildsArray().size());
			assertEquals("19", resp.getDirectory().getId());
			assertEquals("Alanis Morissette", resp.getDirectory().getName());
			for(ChildInfo child : resp.getDirectory().getChildsArray()) {
				if(child.isDir()) 	albums++;
				else				songs++;
			}
			assertEquals(expectedAlbums, albums);
			assertEquals(expectedSongs, songs);
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	/**
	 * @see net.subclient.subsonic.responses.GetMusicDirectoryResponse
	 */
	@Test
	public void testGetMusicDirectoryResponseSingleAlbum() {
		String mockFile = "GetMusicDirectoryResponseSingleAlbumMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetMusicDirectoryResponse resp = this.gson.fromJson(responseText, GetMusicDirectoryResponse.class);
			
			//Assertions
			int albums 			= 0,
					expectedAlbums	= 1,
					songs 			= 0,
					expectedSongs 	= 0,
					expectedSize 	= expectedSongs + expectedAlbums;
			assertEquals(expectedSize, resp.getDirectory().getChildsArray().size());
			assertEquals("19", resp.getDirectory().getId());
			assertEquals("Alanis Morissette", resp.getDirectory().getName());
			for(ChildInfo child : resp.getDirectory().getChildsArray()) {
				if(child.isDir()) 	albums++;
				else				songs++;
			}
			assertEquals(expectedAlbums, albums);
			assertEquals(expectedSongs, songs);
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	/**
	 * @see net.subclient.subsonic.responses.GetMusicDirectoryResponse
	 */
	@Test
	public void testGetMusicDirectoryResponseSingleSong() {
		String mockFile = "GetMusicDirectoryResponseSingleSongMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetMusicDirectoryResponse resp = this.gson.fromJson(responseText, GetMusicDirectoryResponse.class);
			
			//Assertions
			int albums 			= 0,
				expectedAlbums	= 0,
				songs 			= 0,
				expectedSongs 	= 1,
				expectedSize 	= expectedSongs + expectedAlbums;
			assertEquals(expectedSize, resp.getDirectory().getChildsArray().size());
			assertEquals("19", resp.getDirectory().getId());
			assertEquals("Alanis Morissette", resp.getDirectory().getName());
			for(ChildInfo child : resp.getDirectory().getChildsArray()) {
				if(child.isDir()) 	albums++;
				else				songs++;
			}
			assertEquals(expectedAlbums, albums);
			assertEquals(expectedSongs, songs);
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	
	/**
	 * @see net.subclient.subsonic.responses.GetAlbumsResponse
	 */
	@Test
	public void testGetAlbumsResponse() {
		String mockFile = "GetAlbumsResponseMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetAlbumsResponse resp = this.gson.fromJson(responseText, GetAlbumsResponse.class);
			
			//Assertions
			int expectedAlbums 	= 10,
				i				= 0;
			String[] artists = new String[] {
					"virt, Freaky DNA and Norrin Radd", "Austin Wintory", "Paul Oakenfold", "Various Artists", "Moby", "Rob Zombie", "Calvin Carris", "Paul Van Dyk", "Deadmau5", "Tatu"
			};
			assertEquals(expectedAlbums, resp.getAlbums().getAlbumsArray().size());
			for(ChildInfo child : resp.getAlbums().getAlbumsArray()) {
				assertEquals(artists[i], child.getArtist());
				i++;
			}
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	/**
	 * @see net.subclient.subsonic.responses.GetAlbumsResponse
	 */
	@Test
	public void testGetAlbumsResponseSingle() {
		String mockFile = "GetAlbumsResponseSingleMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetAlbumsResponse resp = this.gson.fromJson(responseText, GetAlbumsResponse.class);
			
			//Assertions
			int expectedAlbums = 1;
			assertEquals(expectedAlbums, resp.getAlbums().getAlbumsArray().size());
			assertEquals("Foo Fighters", resp.getAlbums().getAlbumsArray().get(0).getArtist());
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	
	/**
	 * @see net.subclient.subsonic.responses.GetRandomSongsResponse
	 */
	@Test
	public void testGetRandomSongsResponse() {
		String mockFile = "GetRandomSongsResponseMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetRandomSongsResponse resp = this.gson.fromJson(responseText, GetRandomSongsResponse.class);
			
			//Assertions
			int expectedSongs 	= 10,
				i				= 0;
			String[] artists = new String[] {
				"Josh Whelchel", "Evanescence", "virt, Freaky DNA and Norrin Radd", "Linkin Park", "Laura Shigihara", "Marilyn Manson", "Deadmau5", "Binary Finary", "Evanescence", "Evanescence"
			};
			assertEquals(expectedSongs, resp.getRandomSongs().getRandomSongsArray().size());
			for(ChildInfo child : resp.getRandomSongs().getRandomSongsArray()) {
				assertEquals(artists[i], child.getArtist());
				i++;
			}
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	/**
	 * @see net.subclient.subsonic.responses.GetRandomSongsResponse
	 */
	@Test
	public void testGetRandomSongsResponseSingle() {
		String mockFile = "GetRandomSongsResponseSingleMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetRandomSongsResponse resp = this.gson.fromJson(responseText, GetRandomSongsResponse.class);
			
			//Assertions
			int expectedSongs = 1;
			assertEquals(expectedSongs, resp.getRandomSongs().getRandomSongsArray().size());
			assertEquals("Madcore", resp.getRandomSongs().getRandomSongsArray().get(0).getArtist());
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	/**
	 * @see net.subclient.subsonic.responses.GetRandomSongsResponse
	 */
	@Test
	public void testGetRandomSongsResponseEmpty() {
		String mockFile = "GetRandomSongsResponseEmptyMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetRandomSongsResponse resp = this.gson.fromJson(responseText, GetRandomSongsResponse.class);
			
			//Assertions
			int expectedSongs = 0;
			assertEquals(expectedSongs, resp.getRandomSongs().getRandomSongsArray().size());
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	
	/**
	 * @see net.subclient.subsonic.responses.GetPlaylistsResponse
	 */
	@Test
	public void testGetPlaylistsResponse() {
		String mockFile = "GetPlaylistsResponseMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetPlaylistsResponse resp = this.gson.fromJson(responseText, GetPlaylistsResponse.class);
			
			//Assertions
			int expectedPlaylists	= 2,
					i					= 0;
			String[] owners = new String[] {"admin", "alejandro"};
			assertEquals(expectedPlaylists, resp.getPlaylists().getPlaylistsArray().size());
			for(PlaylistInfo playlist : resp.getPlaylists().getPlaylistsArray()) {
				assertEquals(owners[i], playlist.getOwner());
				i++;
			}
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	/**
	 * @see net.subclient.subsonic.responses.GetPlaylistsResponse
	 */
	@Test
	public void testGetPlaylistsResponseSingle() {
		String mockFile = "GetPlaylistsResponseSingleMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetPlaylistsResponse resp = this.gson.fromJson(responseText, GetPlaylistsResponse.class);
			
			//Assertions
			int expectedPlaylists	= 1;
			assertEquals(expectedPlaylists, resp.getPlaylists().getPlaylistsArray().size());
			assertEquals("alejandro", resp.getPlaylists().getPlaylistsArray().get(0).getOwner());
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	/**
	 * @see net.subclient.subsonic.responses.GetPlaylistsResponse
	 */
	@Test
	public void testGetPlaylistsResponseEmpty() {
		String mockFile = "GetPlaylistsResponseEmptyMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetPlaylistsResponse resp = this.gson.fromJson(responseText, GetPlaylistsResponse.class);
			
			//Assertions
			int expectedPlaylists	= 0;
			assertEquals(expectedPlaylists, resp.getPlaylists().getPlaylistsArray().size());
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	
	/**
	 * net.subclient.subsonic.responses.GetPlaylistResponse
	 */
	@Test
	public void testGetPlaylistResponse() {
		String mockFile = "GetPlaylistResponseMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetPlaylistResponse resp = this.gson.fromJson(responseText, GetPlaylistResponse.class);
			
			//Assertions
			int expectedDuration 	= 2927,
				expectedSongCount 	= 14;
			String expectedCreated 	= "2013-04-05T11:27:54",
				   expectedId		= "17",
				   expectedName 	= "Amaranthe. The Nexus",
				   expectedOwner	= "alejandro";
			boolean expectedPublic	= false;
			assertEquals(expectedDuration, resp.getPlaylist().getDuration());
			assertEquals(expectedSongCount, resp.getPlaylist().getSongCount());
			assertEquals(expectedSongCount, resp.getPlaylist().getEntriesArray().size());
			assertEquals(expectedCreated, resp.getPlaylist().getCreationDate());
			assertEquals(expectedId, resp.getPlaylist().getId());
			assertEquals(expectedName, resp.getPlaylist().getName());
			assertEquals(expectedOwner, resp.getPlaylist().getOwner());
			assertEquals(expectedPublic, resp.getPlaylist().isPublic());
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	/**
	 * net.subclient.subsonic.responses.GetPlaylistResponse
	 */
	@Test
	public void testGetPlaylistResponseSingle() {
		String mockFile = "GetPlaylistResponseSingleMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetPlaylistResponse resp = this.gson.fromJson(responseText, GetPlaylistResponse.class);
			
			//Assertions
			int expectedDuration 	= 188,
				expectedSongCount 	= 1;
			String expectedCreated 	= "2013-04-05T11:39:22",
				   expectedId		= "19",
				   expectedName 	= "Single",
				   expectedOwner	= "alejandro";
			boolean expectedPublic	= false;
			assertEquals(expectedDuration, resp.getPlaylist().getDuration());
			assertEquals(expectedSongCount, resp.getPlaylist().getSongCount());
			assertEquals(expectedSongCount, resp.getPlaylist().getEntriesArray().size());
			assertEquals(expectedCreated, resp.getPlaylist().getCreationDate());
			assertEquals(expectedId, resp.getPlaylist().getId());
			assertEquals(expectedName, resp.getPlaylist().getName());
			assertEquals(expectedOwner, resp.getPlaylist().getOwner());
			assertEquals(expectedPublic, resp.getPlaylist().isPublic());
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	/**
	 * net.subclient.subsonic.responses.GetPlaylistResponse
	 */
	@Test
	public void testGetPlaylistResponseEmpty() {
		String mockFile = "GetPlaylistResponseEmptyMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetPlaylistResponse resp = this.gson.fromJson(responseText, GetPlaylistResponse.class);
			
			//Assertions
			int expectedDuration 	= 0,
				expectedSongCount 	= 0;
			String expectedCreated 	= "2013-04-05T11:37:17",
				   expectedId		= "18",
				   expectedName 	= "Empty",
				   expectedOwner	= "alejandro";
			boolean expectedPublic	= false;
			assertEquals(expectedDuration, resp.getPlaylist().getDuration());
			assertEquals(expectedSongCount, resp.getPlaylist().getSongCount());
			assertEquals(expectedSongCount, resp.getPlaylist().getEntriesArray().size());
			assertEquals(expectedCreated, resp.getPlaylist().getCreationDate());
			assertEquals(expectedId, resp.getPlaylist().getId());
			assertEquals(expectedName, resp.getPlaylist().getName());
			assertEquals(expectedOwner, resp.getPlaylist().getOwner());
			assertEquals(expectedPublic, resp.getPlaylist().isPublic());
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	
	/**
	 * @see net.subclient.subsonic.responses.GetPodcastsResponse
	 */
	@Test
	public void testGetPodcastsResponse() {
		String mockFile = "GetPodcastsResponseMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetPodcastsResponse resp = this.gson.fromJson(responseText, GetPodcastsResponse.class);
			
			//Assertions
			int expectedChannels	= 6,
				i					= 0;
			String[] titles = new String[] {
				"Music For Programming", "Corsten's Countdown Official Podcast by FLAIX.fr", "Ti&#235;sto`s Club Life Podcast", "Perfecto Podcast: featuring Paul Oakenfold", 
				"Paul van Dyk's VONYC Sessions Podcast", "A State of Trance Official Podcast"
			};
			assertEquals(expectedChannels, resp.getPodcasts().getChannelsArray().size());
			for(ChannelInfo channel : resp.getPodcasts().getChannelsArray()) {
				assertEquals(titles[i], channel.getTitle());
				i++;
			}
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			e.printStackTrace();
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	/**
	 * @see net.subclient.subsonic.responses.GetPodcastsResponse
	 */
	@Test
	public void testGetPodcastsResponseSingle() {
		String mockFile = "GetPodcastsResponseSingleMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetPodcastsResponse resp = this.gson.fromJson(responseText, GetPodcastsResponse.class);
			
			//Assertions
			assertEquals(1, resp.getPodcasts().getChannelsArray().size());
			assertEquals("A State of Trance Official Podcast", resp.getPodcasts().getChannelsArray().get(0).getTitle());
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			e.printStackTrace();
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	/**
	 * @see net.subclient.subsonic.responses.GetPodcastsResponse
	 */
	@Test
	public void testGetPodcastsResponseEmpty() {
		String mockFile = "GetPodcastsResponseEmptyMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetPodcastsResponse resp = this.gson.fromJson(responseText, GetPodcastsResponse.class);
			
			//Assertions
			assertEquals(0, resp.getPodcasts().getChannelsArray().size());
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	
	/**
	 * net.subclient.subsonic.responses.GetPodcastResponse
	 */
	@Test
	public void testGetPodcastResponse() {
		String mockFile = "GetPodcastResponseMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			GetPodcastResponse resp = this.gson.fromJson(responseText, GetPodcastResponse.class);
			
			//Assertions
			int expectedEpisodesCount 	= 4;
			String expectedId 			= "0",
				   expectedTitle 		= "A State of Trance Official Podcast",
				   expectedStatus 		= "completed",
				   expectedDescription	= "Every week, Armin selects his favourite tunes of the A State of Trance radio show and puts them into his official Podcast. Expect a blend of the hottest in trance and progressive. Enjoy!";
			URL expectedUrl 			= new URL("http://podcast.armadamusic.com/asot/podcast.xml");
			assertEquals(expectedEpisodesCount, resp.getChannel().getEpisodesArray().size());
			assertEquals(expectedId, resp.getChannel().getId());
			assertEquals(expectedTitle, resp.getChannel().getTitle());
			assertEquals(expectedStatus, resp.getChannel().getStatus());
			assertEquals(expectedDescription, resp.getChannel().getDescription());
			assertEquals(expectedUrl, resp.getChannel().getUrl());
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	
	/**
	 * @see net.subclient.subsonic.responses.SearchResponse
	 */
	@Test
	public void testSearchResponse() {
		String mockFile = "SearchResponseMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			SearchResponse resp = this.gson.fromJson(responseText, SearchResponse.class);
			
			//Assertions
			int expectedSongs 	= 5,
				expectedAlbums 	= 4,
				expectedArtists	= 2,
				i				= 0;
			String[] songTitles = new String[] {
				"Paul Oakenfold - Number 116", "Paul Oakenfold - Essential Mix In China", "Remember (Paul)", "Natasha Bedingfield - If You're Gonna Jump (Paul Oakenfold", "Paul Oakenfold - Global Underground - New York - CD1"
			};
			String[] albumTitles = new String[] {
				"Perfecto Podcast- featuring Paul Oakenfold", "Paul van Dyk's VONYC Sessions Podcast", "Paul Van Dyk - (2011) Vonyc Sessions 2011", "Paul Oakenfold - (2012) We are Planet Perfecto Vol 02"
			};
			String[] artists = new String[] {"Paul Oakenfold", "Paul Van Dyk"};
			assertEquals(expectedSongs, resp.getSearchResults().getSongsArray().size());
			assertEquals(expectedAlbums, resp.getSearchResults().getAlbumsArray().size());
			assertEquals(expectedArtists, resp.getSearchResults().getArtistsArray().size());
			for(ChildInfo song : resp.getSearchResults().getSongsArray()) {
				assertEquals(songTitles[i], song.getTitle());
				i++;
			}
			i = 0;
			for(ChildInfo album : resp.getSearchResults().getAlbumsArray()) {
				assertEquals(albumTitles[i], album.getTitle());
				i++;
			}
			i = 0;
			for(FolderInfo artist : resp.getSearchResults().getArtistsArray()) {
				assertEquals(artists[i], artist.getName());
				i++;
			}
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	/**
	 * @see net.subclient.subsonic.responses.SearchResponse
	 */
	@Test
	public void testSearchResponseMixed() {
		String mockFile = "SearchResponseMixedMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			SearchResponse resp = this.gson.fromJson(responseText, SearchResponse.class);
			
			//Assertions
			int expectedSongs 	= 1,
				expectedAlbums 	= 0,
				expectedArtists	= 2;
			assertEquals(expectedSongs, resp.getSearchResults().getSongsArray().size());
			assertEquals(expectedAlbums, resp.getSearchResults().getAlbumsArray().size());
			assertEquals(expectedArtists, resp.getSearchResults().getArtistsArray().size());
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	/**
	 * @see net.subclient.subsonic.responses.SearchResponse
	 */
	@Test
	public void testSearchResponseEmpty() {
		String mockFile = "SearchResponseEmptyMock.json";
		try {
			//Read response file
			String responseText = FileUtils.readTextFile(
				URLDecoder.decode(
					SubsonicResponsesTest.class.getResource("mocks/" + mockFile).getPath(), 
					"UTF-8"
				)
			);
			//Cast response
			SearchResponse resp = this.gson.fromJson(responseText, SearchResponse.class);
			
			//Assertions
			assertEquals(0, resp.getSearchResults().getSongsArray().size());
			assertEquals(0, resp.getSearchResults().getAlbumsArray().size());
			assertEquals(0, resp.getSearchResults().getArtistsArray().size());
		} catch (IOException e) {
			fail(String.format("IOException while opening %s", mockFile));
		} catch (Exception e) {
			fail(String.format("Exception of type %s thrown", e.getClass().getName()));
		}
	}
	
}
